from django.conf import settings

from django_faces.settings import *

class XPavatar:
    def process_response(self, request, response):
        if response.status_code == 200:
            avatar_url = AUTHOR_AVATAR()
            if avatar_url:
                response['X-Pavatar'] = avatar_url
        return response




from md5 import md5
import datetime
import re
import socket
import urllib
import urllib2
import urlparse
import os
import logging
from pdb import set_trace

import Image
try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

from django.db import models
from django.shortcuts import render_to_response, get_object_or_404
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from django.contrib.sites.models import Site

from utils import makeThumb
from django_faces.settings import *


logger = logging.getLogger('avatars.models')

def log_exceptions(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception, e:
            logger.exception('Exception during execution of %r' % func.__name__)
            raise
    wrapper.__name__ = func.__name__
    wrapper.__doc__ = func.__doc__
    return wrapper


class CacheState(models.Model):
    hash = models.CharField( _('Hash'), max_length=32, unique = True)
    enabled = models.BooleanField(_("Avatar enabled"), default=False)
    expire_after = models.DateTimeField(_("Cache expire date"))
    actual_width = models.IntegerField(_('Actual avatar\'s width'), default = 0)
    actual_height = models.IntegerField(_('Actual avatar\'s height'), default = 0)

    def _get_size(self):
        return (self.actual_width, self.actual_height)
    def _set_size(self, size):
        self.actual_width, self.actual_height = size

    size = property(_get_size, _set_size)

    def save(self):
        if self.expire_after is None:
            self.expire_after = datetime.datetime.today() + datetime.timedelta(AVATARS_CACHE_DAYS)
        return super(CacheState, self).save()


def get_pavatar(site, email):
    logger.info('Getting pavatar for site ' + site)
    avatar = None

    if site.startswith('http://%s' % Site.objects.get_current().domain):
        if DONT_FETCH_LOCAL_AVATARS:
            return None

        avatar_url = AUTHOR_AVATAR()
        if avatar_url:
            avatar = urllib2.urlopen(avatar_url)

    if avatar is None:
        source = urllib2.urlopen(site)

        try:
            url = source.info()['X-Pavatar']
            avatar = urllib2.urlopen(url)
        except (KeyError, urllib2.URLError):
            if source.info().subtype == 'html':
                regex = re.compile('<link rel="pavatar" href="([^"]+)" ?/?>', re.I)

                for line in source:
                    m = regex.search(line)
                    if m != None:
                        url = m.group(1)
                        try:
                            avatar = urllib2.urlopen(url)
                        except urllib2.URLError:
                            pass
                        break

    if avatar is None:
        protocol, host, path, _, _  = urlparse.urlsplit(site)
        if path == '':
            path = '/'
        if path[-1] != '/':
            pos = path.rfind('/')
            if pos != -1:
                path = path[:pos+1]
        urls_to_try = (
            urlparse.urlunsplit((protocol, host, path + 'pavatar.png', '', '')),
            urlparse.urlunsplit((protocol, host, 'pavatar.png', '', '')),
        )
        for url in urls_to_try:
            try:
                avatar = urllib2.urlopen(url)
                break
            except urllib2.URLError:
                pass
    return avatar


def get_favicon(site, email):
    logger.info('Getting favicon for site ' + site)
    source = urllib2.urlopen(site)

    avatar = None

    if source.info().subtype == 'html':
        regex = re.compile('<link rel="icon" href="([^"]+)".*/?>', re.I)
        try:
            for line in source:
                m = regex.search(line)
                if m != None:
                    url = m.group(1)
                    if url:
                        if not url.startswith('http'):
                            url = urlparse.urljoin(site, url)

                        try:
                            avatar = urllib2.urlopen(url)
                        except urllib2.URLError:
                            pass
                        break
        except socket.timeout:
            pass

    if avatar is None:
        protocol, host, path, _, _  = urlparse.urlsplit(site)
        if path == '':
            path = '/'
        if path[-1] != '/':
            pos = path.rfind('/')
            if pos != -1:
                path = path[:pos+1]

        urls_to_try = (
                urlparse.urlunsplit((protocol, host, path + 'favicon.png', '', '')),
                urlparse.urlunsplit((protocol, host, path + 'favicon.ico', '', '')),
                urlparse.urlunsplit((protocol, host, 'favicon.png', '', '')),
                urlparse.urlunsplit((protocol, host, 'favicon.ico', '', '')),
        )
        for url in urls_to_try:
            try:
                avatar = urllib2.urlopen(url)
                break
            except urllib2.URLError:
                pass
    return avatar

def get_gravatar(site, email):
    logger.info('Getting gravatar for email ' + email)
    fake_url = 'http://example.com'
    gravatar_options = {
            'size': str(AVATAR_SIZE[0]),
            'default': DEFAULT_GRAVATAR or fake_url,
    }
    hash = md5(email.lower()).hexdigest()
    url = 'http://www.gravatar.com/avatar/%s?%s' % (hash, urllib.urlencode(gravatar_options))

    avatar = urllib2.urlopen(url)

    if avatar and avatar.geturl() != fake_url:
        return avatar
    return None


def get_default(site, email):
    logger.info('Getting default avatar')
    avatar_url = DEFAULT_AVATAR()
    if avatar_url:
        return urllib2.urlopen(avatar_url)
    return None


def fetch_and_save_avatar(avatar, cache):
    if avatar:
        path = os.path.join(settings.MEDIA_ROOT, AVATARS_CACHE_DIR)
        if not os.path.exists(path):
            os.makedirs(path)
        file = open(os.path.join(path, cache.hash + '.png'), 'wb')
        try:
            logging.debug('retriving avatar from %r' % avatar.geturl())
            data = StringIO(avatar.read())
            try:
                image = Image.open(data)
            except IOError, e:
                logger.error('IOError when getting avatar for %s: %s' \
                        % (avatar.geturl(), e))
                cache.save()
                return

            orig_format = image.format
            thumb, cache.size = makeThumb(image, AVATAR_SIZE)
            try:
                logging.debug('original image format is %r, but I\'ll save it as PNG.' % orig_format)
                thumb.save(file, 'PNG')
            except Exception, e:
                logger.error(e)
            cache.enabled = True
        finally:
            file.close()
    cache.save()

def get_avatar(cache, site, email):
    cache.enabled = False
    cache.expire_after = None

    if site is None or site == '':
        cache.save()
        return

    avatar_retrivers = {
        'pavatar': get_pavatar,
        'gravatar': get_gravatar,
        'favicon': get_favicon,
        'default': get_default,
    }

    logging.debug('Using following avatar discovery order: %r' % (AVATAR_DISCOVERY_ORDER,))
    for retriver_name in AVATAR_DISCOVERY_ORDER:
        try:
            avatar = avatar_retrivers.get(retriver_name, get_default)(site, email)
            if avatar:
                fetch_and_save_avatar(avatar, cache)
                if cache.enabled:
                    return
        except Exception:
            logger.exception('exception during avatar retriving')


def gen_hash(email, site):
    return md5(email.lower() + site.lower()).hexdigest()


def get_avatar_url(email, site):
    hash = gen_hash(email, site)
    try:
        cache = CacheState.objects.get(hash=hash)
        if cache.expire_after <= datetime.datetime.today():
            logger.debug('cache for email=%s, site=%s, hash=%s is expired' % (email, site, hash))
            get_avatar(cache, site, email)

    except CacheState.DoesNotExist:
        cache = CacheState(hash=hash)
        get_avatar(cache, site, email)

    cache.save()
    if cache.enabled:
        return (urlparse.urljoin(settings.MEDIA_URL, os.path.join(AVATARS_CACHE_DIR, hash + '.png')),
                dict(width = cache.actual_width, height = cache.actual_height))
    return (None, dict(width=0, height=0))

def comment_postsave(sender, instance):
    hash = gen_hash(instance.author_email, instance.author_site)
    try:
        cache = CacheState.objects.get(hash=hash)
        cache.expire_after = datetime.datetime.today()
        cache.save()
    except CacheState.DoesNotExist:
        pass

if __name__ == 'avatars.models':
    from django.dispatch import dispatcher
    from django.db.models import signals
    try:
        from lfcomments.models import Comment
        dispatcher.connect(comment_postsave, signal=signals.post_save, sender=Comment)
    except Exception, e:
        logging.warning('post_save hook was disabled because of exception: %s' % e)




import urlparse

from django.template import Library

from django_faces.models import get_avatar_url
from django_faces.settings import *

register = Library()

@register.simple_tag
def pavatar_html_link():
    return '<link rel="pavatar" href="%s" />' % AUTHOR_AVATAR()

@register.inclusion_tag('faces/author_block.html')
def author_block():
    return {
        'avatar_url': AUTHOR_AVATAR(),
        'author_name': AUTHOR_NAME,
        'contacts_url': CONTACTS_URL,
    }

@register.inclusion_tag('faces/avatar.html')
def avatar(email, site):
    url, size = get_avatar_url(email, site)
    return {
        'url': url,
        'size': size,
    }



from django.utils.translation import ugettext_lazy as _
from south.db import db
from django.db import models
from django_faces.models import *

class Migration:
    
    def forwards(self):
        
        # Model 'CacheState'
        db.create_table('django_faces_cachestate', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('hash', models.CharField( _('Hash'), max_length=32, unique = True)),
            ('enabled', models.BooleanField(_("Avatar enabled"), default=False)),
            ('expire_after', models.DateTimeField(_("Cache expire date"))),
            ('actual_width', models.IntegerField(_('Actual avatar\'s width'), default = 0)),
            ('actual_height', models.IntegerField(_('Actual avatar\'s height'), default = 0)),
        ))
        
        db.send_create_signal('django_faces', ['CacheState'])
    
    def backwards(self):
        db.delete_table('django_faces_cachestate')
        



import Image

def makeThumb(image, maxSize, method = 3): 
    'Resize PIL image, to fit into the maxSize'

    orig_size = image.size
    if orig_size[0] > maxSize[0] or orig_size[1] > maxSize[1]:
        min_extent = min( orig_size[0], orig_size[1] )
        left = (orig_size[0] - min_extent) / 2 
        top = (orig_size[1] - min_extent) / 2 
        result = image.crop( (left, top, left + min_extent, top + min_extent) )
        result = result.resize( maxSize, Image.ANTIALIAS )
        return (result, maxSize)
    else:
        return (image, orig_size)



import urlparse

from django.conf import settings
from django.contrib.sites.models import Site

def _get_url(settings_name):
    def _getter():
        url = getattr(settings, settings_name, None)
        if url is None or url.startswith('http'):
            return url

        media_url = settings.MEDIA_URL
        if not media_url.startswith('http'):
            media_url = urlparse.urljoin('http://' + Site.objects.get_current().domain, media_url)
        return urlparse.urljoin(media_url, url)
    return _getter

DONT_FETCH_LOCAL_AVATARS = getattr(settings, 'DONT_FETCH_LOCAL_AVATARS', False)
AVATARS_CACHE_DAYS = getattr(settings, 'AVATARS_CACHE_DAYS', 1)
AVATARS_CACHE_DIR = getattr(settings, 'AVATARS_CACHE_DIR', 'avatars_cache')

AVATAR_SIZE = getattr(settings, 'AVATAR_SIZE', (50, 50))
if isinstance(AVATAR_SIZE, int):
    AVATAR_SIZE = (AVATAR_SIZE, AVATAR_SIZE)

AUTHOR_AVATAR = _get_url('AUTHOR_AVATAR')
DEFAULT_AVATAR = _get_url('DEFAULT_AVATAR')
AUTHOR_NAME = getattr(settings, 'AUTHOR_NAME', None)
CONTACTS_URL = getattr(settings, 'CONTACTS_URL', None)
AVATAR_DISCOVERY_ORDER = getattr(settings, 'AVATAR_DISCOVERY_ORDER', ('pavatar', 'gravatar', 'favicon', 'default'))
# also, it can be 'monsterid' or 'identicon' or 'wavatar'
DEFAULT_GRAVATAR = getattr(settings, 'DEFAULT_GRAVATAR', None)
